/*
  ******************************************************************************
  * @file    spi.c
  * @author  Angel_YY
  * @version V1.0.0
  * @date    2021
  * @brief   spi demo source code.
  ******************************************************************************
*/
 
#include "n32g45x_spi.h"
#include "spi.h"

/*
 * SPI2_FRAM_CS   -> PB12
 * SPI2_FRAM_SCK  -> PB13
 * SPI2_FRAM_MISO -> PB14
 * SPI2_FRAM_MOSI -> PB15
 */
 
 
//#define FM25L04B

SPI_InitType SPI_InitStructure;

void SPI_Delay_us(uint32_t nCount)
{
    uint32_t tcnt;
    while (nCount--)
    {
        tcnt = 144 / 5;
        while (tcnt--){;}
    }
}
/**
 * @brief  Configures the different SPIy and SPIz GPIO ports.
 * @param SPIy_Mode Specifies the SPIy operating mode.
 *            This parameter can be:
 *              -  SPIy_Mode_Master
 *              -  SPIy_Mode_Slave
 * @param SPIz_Mode Specifies the SPIz operating mode.
 *            This parameter can be:
 *              -  SPIz_Mode_Master
 *              -  SPIz_Mode_Slave
 */
void GPIO_Configuration(uint16_t SPIy_Mode, uint16_t SPIz_Mode)
{
    GPIO_InitType GPIO_InitStructure;

    /* Configure SPIz pins: SCK, MISO and MOSI ---------------------------------*/
    GPIO_InitStructure.Pin = SPIz_PIN_SCK | SPIz_PIN_MOSI;

    if (SPIz_Mode == SPI_MODE_SLAVE)
    {
        /* Configure SCK and MOSI pins as Input Floating */
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    }
    else
    {
        /* Configure SCK and MOSI pins as Alternate Function Push Pull */
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    }
    GPIO_InitPeripheral(SPIz_GPIO, &GPIO_InitStructure);

    GPIO_InitStructure.Pin = SPIz_PIN_MISO;
    if (SPIz_Mode == SPI_MODE_SLAVE)
    {
        /* Configure MISO pin as Alternate Function Push Pull */
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    }
    else
    { /* Configure MISO pin as Input Floating  */
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    }
    GPIO_InitPeripheral(SPIz_GPIO, &GPIO_InitStructure);
}
                                      
int spi_all_master_init(void)
{
    GPIO_InitType GPIO_InitStructure;
    
    /* Enable SPIy clock and GPIO clock for SPIy and SPIz */
    RCC_EnableAPB2PeriphClk(SPIz_GPIO_CLK , ENABLE);

    /* Enable SPIz Periph clock */
    RCC_EnableAPB1PeriphClk(SPIz_CLK, ENABLE);
    
    /* 1st phase: SPIy Master and SPIz Slave */
    /* GPIO configuration ------------------------------------------------------*/
    GPIO_Configuration(SPI_MODE_MASTER, SPI_MODE_MASTER);
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.Pin        = GPIO_PIN_12;
    GPIO_InitPeripheral(GPIOB, &GPIO_InitStructure);

    GPIO_WriteBit(GPIOB,GPIO_PIN_12,Bit_SET);
    /* SPIz Config -------------------------------------------------------------*/
    SPI_InitStructure.DataDirection = SPI_DIR_DOUBLELINE_FULLDUPLEX;
    SPI_InitStructure.SpiMode       = SPI_MODE_MASTER;
    SPI_InitStructure.DataLen       = SPI_DATA_SIZE_8BITS;
    SPI_InitStructure.CLKPOL        = SPI_CLKPOL_LOW;
    SPI_InitStructure.CLKPHA        = SPI_CLKPHA_FIRST_EDGE;
    SPI_InitStructure.NSS           = SPI_NSS_SOFT;
    SPI_InitStructure.BaudRatePres  = SPI_BR_PRESCALER_4;
    SPI_InitStructure.FirstBit      = SPI_FB_MSB;//SPI_FB_LSB;
    SPI_InitStructure.CRCPoly       = 7;
    SPI_Init(SPIz, &SPI_InitStructure);
    /* Enable SPIz */
    SPI_Enable(SPIz, ENABLE);
    
    return 0;
}
void fram_cs_low(void)
{
    GPIO_WriteBit(GPIOB,GPIO_PIN_12,Bit_RESET);
}
void fram_cs_high(void)
{
    GPIO_WriteBit(GPIOB,GPIO_PIN_12,Bit_SET);
}

int spi_transmit_data(SPI_Module* SPIx,char *pdata,int len)
{
    int TxIdx=0;

    /* Send SPIz data */
    while(TxIdx < len)
    {
        /* Wait for SPIx Tx buffer empty */
        while (SPI_I2S_GetStatus(SPIx, SPI_I2S_TE_FLAG) == RESET)
        ;
        SPI_I2S_TransmitData(SPIx, pdata[TxIdx]);
        /* Wait for SPIx Tx buffer empty */
        while (SPI_I2S_GetStatus(SPIx, SPI_I2S_RNE_FLAG) == RESET)
        ;
        SPI_I2S_ReceiveData(SPIx);
        
        TxIdx++;
    }   
    return 0;    
}
int spi_receive_data(SPI_Module* SPIx,char *pdata,int len)
{
    int TxIdx=0;
    while(TxIdx < len)
    {
        /* Wait for SPIx Tx buffer empty */
        while (SPI_I2S_GetStatus(SPIx, SPI_I2S_TE_FLAG) == RESET)
        ;
        SPI_I2S_TransmitData(SPIx, 0xff);
        /* Wait for SPIx Tx buffer empty */
        while (SPI_I2S_GetStatus(SPIx, SPI_I2S_RNE_FLAG) == RESET)
        ;
        pdata[TxIdx] = SPI_I2S_ReceiveData(SPIx);
        
        TxIdx++;
    } 
    return 0;    
}                                      

int InitFRAM(void){
	//SPI_Init(1);
	return 0;
}

int Fram_WriteData(unsigned short addr,unsigned char * pData,int len)
{
    uint8_t sendBuff[20];
    uint8_t recvBuff[20];
    int byteCount=0,i=0;
    if(len>16)
    {
        return -1;  
    }
    sendBuff[0] = 0x06;  
    fram_cs_low();
    spi_transmit_data(SPIz,(char *)sendBuff,1);

    fram_cs_high();
    fram_cs_low();
    sendBuff[0] = 0x02;
    sendBuff[1] = addr>>8;
    sendBuff[2] = addr&0xff;
    for(i=0;i<len;i++)
    {
        sendBuff[3+i]=pData[i];
    }

    i=100; 
    byteCount   = len+3; 
    spi_transmit_data(SPIz,(char *)sendBuff,byteCount);
    fram_cs_high();
    return 0;
}

int Fram_ReadData(unsigned short addr,unsigned char *pData,int len)
{
    uint8_t sendBuff[20];
    uint8_t recvBuff[20];
    int byteCount=0,i=0;
    if(len>16)
    {
        return -1;
    }

    sendBuff[0] = 0x03;
    sendBuff[1] = addr>>8;
    sendBuff[2] = addr&0xff;
    byteCount   = len+3;

    fram_cs_low();
    spi_transmit_data(SPIz,(char *)sendBuff,3);
    
    spi_receive_data(SPIz,(char *)pData,len);
    fram_cs_high();
    return 0;
}
